#version 430 core
/*
* Copyright 2010 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

out vec4 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform uint hasLightmap;
uniform uint isLava;
uniform sampler2D lightmap;
uniform vec3 lavacolor1;
uniform vec3 lavacolor2;
uniform float brightness;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight1 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL1;

uniform struct PointLight2 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL2;

uniform struct PointLight3 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL3;


uniform struct PointLight4 {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL4;

float rand(vec2 n) {
	return fract(sin(dot(n, vec2(12.9898, 4.1414))) * 43758.5453);
}

float noise(vec2 p){
	vec2 ip = floor(p);
	vec2 u = fract(p);
	u = u*u*(3.0-2.0*u);

	float res = mix(
		mix(rand(ip),rand(ip+vec2(1.0,0.0)),u.x),
		mix(rand(ip+vec2(0.0,1.0)),rand(ip+vec2(1.0,1.0)),u.x),u.y);
	return res*res;
}

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att;
}

void main() {
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	vec3 texColor;

  if(isLava==0){
		if(hasLightmap==1){
		texColor = texture(diffuseTexture, vert.uv).rgb * texture(lightmap, vert.uv).rgb;
		color = vec4(texColor * materialCoefficients.x, 1); // ambient
		}else{
		texColor = texture(diffuseTexture, vert.uv).rgb;
		color = vec4(texColor * materialCoefficients.x, 1); // ambient
		}
  }
	if(isLava == 1){
	vec3 color1 = lavacolor1;//vec3(100,20,0);
	vec3 color2 = lavacolor2;//vec3(200,170,5);
	float procInt = (1 + sin(vert.uv.x + (noise(vec2(vert.uv.x * 25, vert.uv.y * 25))/1) * 10)) / 5;

	texColor = mix(color1,color2,procInt) * procInt - 0.1;
	color = vec4(texColor * materialCoefficients.x, 1);
}
	// add directional light contribution
	//color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));

	// add point light contribution
	color.rgb += phong(n, pointL1.position - vert.position_world, v, pointL1.color * texColor, materialCoefficients.y, pointL1.color, materialCoefficients.z, specularAlpha, true, pointL1.attenuation);
	color.rgb += phong(n, pointL2.position - vert.position_world, v, pointL2.color * texColor, materialCoefficients.y, pointL2.color, materialCoefficients.z, specularAlpha, true, pointL2.attenuation);
	color.rgb += phong(n, pointL3.position - vert.position_world, v, pointL3.color * texColor, materialCoefficients.y, pointL3.color, materialCoefficients.z, specularAlpha, true, pointL3.attenuation);
	color.rgb += phong(n, pointL4.position - vert.position_world, v, pointL4.color * texColor, materialCoefficients.y, pointL4.color, materialCoefficients.z, specularAlpha, true, pointL4.attenuation);

	color.rgb = color.rgb * brightness;
}
